@@ -1,27 +1,36 @@
-/*
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2004-2023 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
-Copyright (C) 2004-2015 John W. Eaton
-Copyright (C) 2010 VZLU Prague
+// FIXME: Do not uncomment these lines to have this file included only once.
+//        The build will break (2/6/2016).
+// #if ! defined (octave_ov_intx_h)
+// #define octave_ov_intx_h 1
 
-This file is part of Octave.
+#include "octave-config.h"
 
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
-
-#include <stdlib.h>
+#include <cstdlib>
 
 #include <iosfwd>
 #include <string>
@@ -35,7 +44,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "ov-base.h"
 #include "ov-base-int.h"
 #include "ov-typeinfo.h"
-#include "gripes.h"
+#include "errwarn.h"
 
 #include "ov-re-mat.h"
 #include "ov-scalar.h"
@@ -43,96 +52,90 @@ along with Octave; see the file COPYING.  If not, see
 class
 OCTINTERP_API
 OCTAVE_VALUE_INT_MATRIX_T
-  : public octave_base_int_matrix<intNDArray<OCTAVE_INT_T> >
+  : public octave_base_int_matrix<intNDArray<OCTAVE_INT_T>>
 {
 public:
 
   OCTAVE_VALUE_INT_MATRIX_T (void)
-    : octave_base_int_matrix<intNDArray<OCTAVE_INT_T> > () { }
+    : octave_base_int_matrix<intNDArray<OCTAVE_INT_T>> () { }
 
   OCTAVE_VALUE_INT_MATRIX_T (const intNDArray<OCTAVE_INT_T>& nda)
-    : octave_base_int_matrix<intNDArray<OCTAVE_INT_T> > (nda) { }
+    : octave_base_int_matrix<intNDArray<OCTAVE_INT_T>> (nda) { }
 
   OCTAVE_VALUE_INT_MATRIX_T (const Array<OCTAVE_INT_T>& nda)
-    : octave_base_int_matrix<intNDArray<OCTAVE_INT_T> >
-        (intNDArray<OCTAVE_INT_T> (nda)) { }
+    : octave_base_int_matrix<intNDArray<OCTAVE_INT_T>>
+      (intNDArray<OCTAVE_INT_T> (nda)) { }
 
-  ~OCTAVE_VALUE_INT_MATRIX_T (void) { }
+  ~OCTAVE_VALUE_INT_MATRIX_T (void) = default;
 
-  octave_base_value *clone (void) const
+  octave_base_value * clone (void) const
   { return new OCTAVE_VALUE_INT_MATRIX_T (*this); }
 
-  octave_base_value *empty_clone (void) const
+  octave_base_value * empty_clone (void) const
   { return new OCTAVE_VALUE_INT_MATRIX_T (); }
 
   bool OCTAVE_TYPE_PREDICATE_FUNCTION (void) const { return true; }
 
-  bool is_integer_type (void) const { return true; }
+  bool isinteger (void) const { return true; }
 
   builtin_type_t builtin_type (void) const { return OCTAVE_INT_BTYP; }
 
 public:
 
   int8NDArray
-  int8_array_value (void) const { return int8NDArray (matrix); }
+  int8_array_value (void) const { return int8NDArray (m_matrix); }
 
   int16NDArray
-  int16_array_value (void) const { return int16NDArray (matrix); }
+  int16_array_value (void) const { return int16NDArray (m_matrix); }
 
   int32NDArray
-  int32_array_value (void) const { return int32NDArray (matrix); }
+  int32_array_value (void) const { return int32NDArray (m_matrix); }
 
   int64NDArray
-  int64_array_value (void) const { return int64NDArray (matrix); }
+  int64_array_value (void) const { return int64NDArray (m_matrix); }
 
   uint8NDArray
-  uint8_array_value (void) const { return uint8NDArray (matrix); }
+  uint8_array_value (void) const { return uint8NDArray (m_matrix); }
 
   uint16NDArray
-  uint16_array_value (void) const { return uint16NDArray (matrix); }
+  uint16_array_value (void) const { return uint16NDArray (m_matrix); }
 
   uint32NDArray
-  uint32_array_value (void) const { return uint32NDArray (matrix); }
+  uint32_array_value (void) const { return uint32NDArray (m_matrix); }
 
   uint64NDArray
-  uint64_array_value (void) const { return uint64NDArray (matrix); }
+  uint64_array_value (void) const { return uint64NDArray (m_matrix); }
 
   double
   double_value (bool = false) const
   {
-    double retval = lo_ieee_nan_value ();
+    double retval;
 
-    if (numel () > 0)
-      {
-        gripe_implicit_conversion ("Octave:array-to-scalar",
-                                   type_name (), "real scalar");
+    if (isempty ())
+      err_invalid_conversion (type_name (), "real scalar");
 
-        retval = matrix(0).double_value ();
-      }
-    else
-      gripe_invalid_conversion (type_name (), "real scalar");
+    warn_implicit_conversion ("Octave:array-to-scalar",
+                              type_name (), "real scalar");
+
+    retval = m_matrix(0).double_value ();
 
     return retval;
-
   }
 
   float
   float_value (bool = false) const
   {
-    float retval = lo_ieee_float_nan_value ();
+    float retval;
 
-    if (numel () > 0)
-      {
-        gripe_implicit_conversion ("Octave:array-to-scalar",
-                                   type_name (), "real scalar");
+    if (isempty ())
+      err_invalid_conversion (type_name (), "real scalar");
 
-        retval = matrix(0).float_value ();
-      }
-    else
-      gripe_invalid_conversion (type_name (), "real scalar");
+    warn_implicit_conversion ("Octave:array-to-scalar",
+                              type_name (), "real scalar");
+
+    retval = m_matrix(0).float_value ();
 
     return retval;
-
   }
 
   double scalar_value (bool = false) const { return double_value (); }
@@ -144,16 +147,15 @@ public:
   {
     Matrix retval;
     dim_vector dv = dims ();
-    if (dv.length () > 2)
+    if (dv.ndims () > 2)
       error ("invalid conversion of %s to Matrix", type_name ().c_str ());
-    else
-      {
-        retval = Matrix (dv(0), dv(1));
-        double *vec = retval.fortran_vec ();
-        octave_idx_type nel = matrix.numel ();
-        for (octave_idx_type i = 0; i < nel; i++)
-          vec[i] = matrix(i).double_value ();
-      }
+
+    retval = Matrix (dv(0), dv(1));
+    double *vec = retval.fortran_vec ();
+    octave_idx_type nel = m_matrix.numel ();
+    for (octave_idx_type i = 0; i < nel; i++)
+      vec[i] = m_matrix(i).double_value ();
+
     return retval;
   }
 
@@ -162,16 +164,15 @@ public:
   {
     FloatMatrix retval;
     dim_vector dv = dims ();
-    if (dv.length () > 2)
+    if (dv.ndims () > 2)
       error ("invalid conversion of %s to FloatMatrix", type_name ().c_str ());
-    else
-      {
-        retval = FloatMatrix (dv(0), dv(1));
-        float *vec = retval.fortran_vec ();
-        octave_idx_type nel = matrix.numel ();
-        for (octave_idx_type i = 0; i < nel; i++)
-          vec[i] = matrix(i).float_value ();
-      }
+
+    retval = FloatMatrix (dv(0), dv(1));
+    float *vec = retval.fortran_vec ();
+    octave_idx_type nel = m_matrix.numel ();
+    for (octave_idx_type i = 0; i < nel; i++)
+      vec[i] = m_matrix(i).float_value ();
+
     return retval;
   }
 
@@ -180,16 +181,15 @@ public:
   {
     ComplexMatrix retval;
     dim_vector dv = dims ();
-    if (dv.length () > 2)
+    if (dv.ndims () > 2)
       error ("invalid conversion of %s to Matrix", type_name ().c_str ());
-    else
-      {
-        retval = ComplexMatrix (dv(0), dv(1));
-        Complex *vec = retval.fortran_vec ();
-        octave_idx_type nel = matrix.numel ();
-        for (octave_idx_type i = 0; i < nel; i++)
-          vec[i] = Complex (matrix(i).double_value ());
-      }
+
+    retval = ComplexMatrix (dv(0), dv(1));
+    Complex *vec = retval.fortran_vec ();
+    octave_idx_type nel = m_matrix.numel ();
+    for (octave_idx_type i = 0; i < nel; i++)
+      vec[i] = Complex (m_matrix(i).double_value ());
+
     return retval;
   }
 
@@ -198,60 +198,59 @@ public:
   {
     FloatComplexMatrix retval;
     dim_vector dv = dims ();
-    if (dv.length () > 2)
+    if (dv.ndims () > 2)
       error ("invalid conversion of %s to FloatMatrix", type_name ().c_str ());
-    else
-      {
-        retval = FloatComplexMatrix (dv(0), dv(1));
-        FloatComplex *vec = retval.fortran_vec ();
-        octave_idx_type nel = matrix.numel ();
-        for (octave_idx_type i = 0; i < nel; i++)
-          vec[i] = FloatComplex (matrix(i).float_value ());
-      }
+
+    retval = FloatComplexMatrix (dv(0), dv(1));
+    FloatComplex *vec = retval.fortran_vec ();
+    octave_idx_type nel = m_matrix.numel ();
+    for (octave_idx_type i = 0; i < nel; i++)
+      vec[i] = FloatComplex (m_matrix(i).float_value ());
+
     return retval;
   }
 
   NDArray
   array_value (bool = false) const
   {
-    NDArray retval (matrix.dims ());
+    NDArray retval (m_matrix.dims ());
     double *vec = retval.fortran_vec ();
-    octave_idx_type nel = matrix.numel ();
+    octave_idx_type nel = m_matrix.numel ();
     for (octave_idx_type i = 0; i < nel; i++)
-      vec[i] = matrix(i).double_value ();
+      vec[i] = m_matrix(i).double_value ();
     return retval;
   }
 
   FloatNDArray
   float_array_value (bool = false) const
   {
-    FloatNDArray retval (matrix.dims ());
+    FloatNDArray retval (m_matrix.dims ());
     float *vec = retval.fortran_vec ();
-    octave_idx_type nel = matrix.numel ();
+    octave_idx_type nel = m_matrix.numel ();
     for (octave_idx_type i = 0; i < nel; i++)
-      vec[i] = matrix(i).float_value ();
+      vec[i] = m_matrix(i).float_value ();
     return retval;
   }
 
   ComplexNDArray
   complex_array_value (bool = false) const
   {
-    ComplexNDArray retval (matrix.dims ());
+    ComplexNDArray retval (m_matrix.dims ());
     Complex *vec = retval.fortran_vec ();
-    octave_idx_type nel = matrix.numel ();
+    octave_idx_type nel = m_matrix.numel ();
     for (octave_idx_type i = 0; i < nel; i++)
-      vec[i] = Complex (matrix(i).double_value ());
+      vec[i] = Complex (m_matrix(i).double_value ());
     return retval;
   }
 
   FloatComplexNDArray
   float_complex_array_value (bool = false) const
   {
-    FloatComplexNDArray retval (matrix.dims ());
+    FloatComplexNDArray retval (m_matrix.dims ());
     FloatComplex *vec = retval.fortran_vec ();
-    octave_idx_type nel = matrix.numel ();
+    octave_idx_type nel = m_matrix.numel ();
     for (octave_idx_type i = 0; i < nel; i++)
-      vec[i] = FloatComplex (matrix(i).float_value ());
+      vec[i] = FloatComplex (m_matrix(i).float_value ());
     return retval;
   }
 
@@ -262,12 +261,12 @@ public:
 
     octave_idx_type nel = numel ();
 
-    if (warn && matrix.any_element_not_one_or_zero ())
-      gripe_logical_conversion ();
+    if (warn && m_matrix.any_element_not_one_or_zero ())
+      warn_logical_conversion ();
 
     bool *vec = retval.fortran_vec ();
     for (octave_idx_type i = 0; i < nel; i++)
-      vec[i] = matrix(i).bool_value ();
+      vec[i] = m_matrix(i).bool_value ();
 
     return retval;
   }
@@ -281,7 +280,7 @@ public:
 
     char *vec = retval.fortran_vec ();
     for (octave_idx_type i = 0; i < nel; i++)
-      vec[i] = matrix(i).char_value ();
+      vec[i] = m_matrix(i).char_value ();
 
     return retval;
   }
@@ -302,33 +301,31 @@ public:
     matrix_ref ().changesign ();
   }
 
-  idx_vector index_vector (bool /* require_integers */ = false) const
+  octave::idx_vector index_vector (bool /* require_integers */ = false) const
   {
-    return idx_cache ? *idx_cache : set_idx_cache (idx_vector (matrix));
+    return m_idx_cache ? *m_idx_cache
+           : set_idx_cache (octave::idx_vector (m_matrix));
   }
 
-  int write (octave_stream& os, int block_size,
+  int write (octave::stream& os, int block_size,
              oct_data_conv::data_type output_type, int skip,
-             oct_mach_info::float_format flt_fmt) const
-  { return os.write (matrix, block_size, output_type, skip, flt_fmt); }
+             octave::mach_info::float_format flt_fmt) const
+  { return os.write (m_matrix, block_size, output_type, skip, flt_fmt); }
 
-  // Unsafe.  This function exists to support the MEX interface.
-  // You should not use it anywhere else.
-  void *mex_get_data (void) const { return matrix.mex_get_data (); }
-
-  mxArray *as_mxArray (void) const
+  mxArray * as_mxArray (bool interleaved) const
   {
-    mxArray *retval = new mxArray (OCTAVE_INT_MX_CLASS, dims (), mxREAL);
+    mxArray *retval = new mxArray (interleaved, OCTAVE_INT_MX_CLASS, dims (),
+                                   mxREAL);
 
-    OCTAVE_INT_T::val_type *pr = static_cast<OCTAVE_INT_T::val_type *>
-                                 (retval->get_data ());
+    OCTAVE_INT_T::val_type *pd
+      = static_cast<OCTAVE_INT_T::val_type *> (retval->get_data ());
 
     mwSize nel = numel ();
 
-    const OCTAVE_INT_T *p = matrix.data ();
+    const OCTAVE_INT_T *pdata = m_matrix.data ();
 
     for (mwIndex i = 0; i < nel; i++)
-      pr[i] = p[i].value ();
+      pd[i] = pdata[i].value ();
 
     return retval;
   }
@@ -338,29 +335,29 @@ public:
     switch (umap)
       {
       case umap_abs:
-        return matrix.abs ();
+        return m_matrix.abs ();
       case umap_signum:
-        return matrix.signum ();
+        return m_matrix.signum ();
       case umap_ceil:
       case umap_conj:
       case umap_fix:
       case umap_floor:
       case umap_real:
       case umap_round:
-        return matrix;
+        return m_matrix;
       case umap_imag:
-        return intNDArray<OCTAVE_INT_T> (matrix.dims (), OCTAVE_INT_T ());
+        return intNDArray<OCTAVE_INT_T> (m_matrix.dims (), OCTAVE_INT_T ());
       case umap_isnan:
       case umap_isna:
       case umap_isinf:
-        return boolNDArray (matrix.dims (), false);
-      case umap_finite:
-        return boolNDArray (matrix.dims (), true);
+        return boolNDArray (m_matrix.dims (), false);
+      case umap_isfinite:
+        return boolNDArray (m_matrix.dims (), true);
 
       // Special cases for Matlab compatibility.
       case umap_xtolower:
       case umap_xtoupper:
-        return matrix;
+        return m_matrix;
 
       default:
         {
@@ -372,8 +369,19 @@ public:
       }
   }
 
+  bool save_hdf5 (octave_hdf5_id loc_id, const char *name, bool flag)
+  {
+    return save_hdf5_internal (loc_id, s_hdf5_save_type, name, flag);
+  }
+
+  bool load_hdf5 (octave_hdf5_id loc_id, const char *name)
+  {
+    return load_hdf5_internal (loc_id, s_hdf5_save_type, name);
+  }
+
 private:
 
+  static octave_hdf5_id s_hdf5_save_type;
 
   DECLARE_OV_TYPEID_FUNCTIONS_AND_DATA
 };
@@ -391,12 +399,12 @@ public:
   OCTAVE_VALUE_INT_SCALAR_T (const OCTAVE_INT_T& nda)
     : octave_base_int_scalar<OCTAVE_INT_T> (nda) { }
 
-  ~OCTAVE_VALUE_INT_SCALAR_T (void) { }
+  ~OCTAVE_VALUE_INT_SCALAR_T (void) = default;
 
-  octave_base_value *clone (void) const
+  octave_base_value * clone (void) const
   { return new OCTAVE_VALUE_INT_SCALAR_T (*this); }
 
-  octave_base_value *empty_clone (void) const
+  octave_base_value * empty_clone (void) const
   { return new OCTAVE_VALUE_INT_MATRIX_T (); }
 
   octave_value do_index_op (const octave_value_list& idx,
@@ -416,12 +424,12 @@ public:
     (new OCTAVE_VALUE_INT_MATRIX_T
      (OCTAVE_VALUE_INT_NDARRAY_EXTRACTOR_FUNCTION ()));
 
-    return tmp.do_index_op (idx, resize_ok);
+    return tmp.index_op (idx, resize_ok);
   }
 
   bool OCTAVE_TYPE_PREDICATE_FUNCTION (void) const { return true; }
 
-  bool is_integer_type (void) const { return true; }
+  bool isinteger (void) const { return true; }
 
   builtin_type_t builtin_type (void) const { return OCTAVE_INT_BTYP; }
 
@@ -514,7 +522,7 @@ public:
   matrix_value (bool = false) const
   {
     Matrix retval (1, 1);
-    retval(0,0) = scalar.double_value ();
+    retval(0, 0) = scalar.double_value ();
     return retval;
   }
 
@@ -522,7 +530,7 @@ public:
   float_matrix_value (bool = false) const
   {
     FloatMatrix retval (1, 1);
-    retval(0,0) = scalar.float_value ();
+    retval(0, 0) = scalar.float_value ();
     return retval;
   }
 
@@ -530,7 +538,7 @@ public:
   complex_matrix_value (bool = false) const
   {
     ComplexMatrix retval (1, 1);
-    retval(0,0) = Complex (scalar.double_value ());
+    retval(0, 0) = Complex (scalar.double_value ());
     return retval;
   }
 
@@ -538,7 +546,7 @@ public:
   float_complex_matrix_value (bool = false) const
   {
     FloatComplexMatrix retval (1, 1);
-    retval(0,0) = FloatComplex (scalar.float_value ());
+    retval(0, 0) = FloatComplex (scalar.float_value ());
     return retval;
   }
 
@@ -562,7 +570,7 @@ public:
   complex_array_value (bool = false) const
   {
     ComplexNDArray retval (dim_vector (1, 1));
-    retval(0) = FloatComplex (scalar.double_value ());
+    retval(0) = Complex (scalar.double_value ());
     return retval;
   }
 
@@ -577,7 +585,7 @@ public:
   bool bool_value (bool warn = false) const
   {
     if (warn && scalar != 0.0 && scalar != 1.0)
-      gripe_logical_conversion ();
+      warn_logical_conversion ();
 
     return scalar.bool_value ();
   }
@@ -588,7 +596,7 @@ public:
     boolNDArray retval (dim_vector (1, 1));
 
     if (warn && scalar != 0.0 && scalar != 1.0)
-      gripe_logical_conversion ();
+      warn_logical_conversion ();
 
     retval(0) = scalar.bool_value ();
 
@@ -613,28 +621,26 @@ public:
     scalar -= OCTAVE_INT_T (1);
   }
 
-  idx_vector index_vector (bool /* require_integers */ = false) const { return idx_vector (scalar); }
+  octave::idx_vector index_vector (bool /* require_integers */ = false) const
+  { return octave::idx_vector (scalar); }
 
-  int write (octave_stream& os, int block_size,
-             oct_data_conv::data_type output_type, octave_idx_type skip,
-             oct_mach_info::float_format flt_fmt) const
+  int write (octave::stream& os, int block_size,
+             oct_data_conv::data_type output_type, int skip,
+             octave::mach_info::float_format flt_fmt) const
   {
     return os.write (OCTAVE_VALUE_INT_NDARRAY_EXTRACTOR_FUNCTION (),
                      block_size, output_type, skip, flt_fmt);
   }
 
-  // Unsafe.  This function exists to support the MEX interface.
-  // You should not use it anywhere else.
-  void *mex_get_data (void) const { return scalar.mex_get_data (); }
-
-  mxArray *as_mxArray (void) const
+  mxArray * as_mxArray (bool interleaved) const
   {
-    mxArray *retval = new mxArray (OCTAVE_INT_MX_CLASS, 1, 1, mxREAL);
+    mxArray *retval = new mxArray (interleaved, OCTAVE_INT_MX_CLASS, 1, 1,
+                                   mxREAL);
 
-    OCTAVE_INT_T::val_type *pr = static_cast<OCTAVE_INT_T::val_type *>
-                                 (retval->get_data ());
+    OCTAVE_INT_T::val_type *pd
+      = static_cast<OCTAVE_INT_T::val_type *> (retval->get_data ());
 
-    pr[0] = scalar.value ();
+    pd[0] = scalar.value ();
 
     return retval;
   }
@@ -660,7 +666,7 @@ public:
       case umap_isna:
       case umap_isinf:
         return false;
-      case umap_finite:
+      case umap_isfinite:
         return true;
 
       // Special cases for Matlab compatibility.
@@ -676,8 +682,19 @@ public:
       }
   }
 
+  bool save_hdf5 (octave_hdf5_id loc_id, const char *name, bool flag)
+  {
+    return save_hdf5_internal (loc_id, s_hdf5_save_type, name, flag);
+  }
+
+  bool load_hdf5 (octave_hdf5_id loc_id, const char *name)
+  {
+    return load_hdf5_internal (loc_id, s_hdf5_save_type, name);
+  }
+
 private:
 
+  static octave_hdf5_id s_hdf5_save_type;
 
   DECLARE_OV_TYPEID_FUNCTIONS_AND_DATA
 };
